@@ -6,11 +6,13 @@ from multiprocessing.pool import Pool
 
 import click
 import humanize
+import requests
 from prettytable import PrettyTable
 from pynotifier import Notification
 
 from rebotics_sdk.cli.common import shell, configure, roles
 from .utils import ReboticsCLIContext, app_dir, pass_rebotics_context, read_saved_role, process_role
+from ..advanced.packers import ClassificationDatabasePacker, ClassificationDatabaseException
 from ..providers import AdminProvider, RetailerProvider, sleep
 from ..providers.facenet import FacenetProvider
 from ..utils import Timer
@@ -20,7 +22,7 @@ from ..utils import Timer
 @click.option('-f', '--format', default='table', type=click.Choice(['table', 'id', 'json']), help='Result rendering')
 @click.option('-v', '--verbose', is_flag=True, help='Enables verbose mode')
 @click.option('-c', '--config', type=click.Path(), default='admin.json', help="Specify what config.json to use")
-@click.option('-r', '--role',  default=lambda: read_saved_role('admin'), help="Key to specify what admin to use")
+@click.option('-r', '--role', default=lambda: read_saved_role('admin'), help="Key to specify what admin to use")
 @click.version_option()
 @click.pass_context
 def api(ctx, format, verbose, config, role):
@@ -38,7 +40,7 @@ def api(ctx, format, verbose, config, role):
 
 
 def get_retailer_version_task(retailer_dict):
-    retailer_provider = RetailerProvider(host=retailer_dict['host'], retries=1)
+    retailer_provider = RetailerProvider(host=retailer_dict['host'], retries=1, timeout=5)
     try:
         response = retailer_provider.version()
         version = response['version']
@@ -57,6 +59,17 @@ def get_retailer_version_task(retailer_dict):
     return d
 
 
+@api.command()
+@click.argument('retailer')
+@pass_rebotics_context
+def configurations(ctx, retailer):
+    try:
+        res = ctx.provider.get_configurations(retailer)
+        ctx.format_result(res)
+    except Exception as exc:
+        raise click.ClickException(exc)
+
+
 @api.command()
 @click.option('-n', '--notify', is_flag=True)
 @click.option('-d', '--delay', type=click.INT, default=60)
@@ -205,6 +218,63 @@ def set_retailer_url(ctx, retailer, url):
         click.echo('Set new host for retailer %s' % retailer)
 
 
+def upload_packed(ctx, import_request, packed):
+    destination = import_request['destination']
+    files = {
+        'file': ('features_backup.rcdb', packed)
+    }
+    response = requests.post(destination['url'], data=destination['fields'], files=files)
+    if response.status_code < 300:
+        ctx.provider.notify_classification_database_import_done(
+            id=import_request['id']
+        )
+        click.echo('File was uploaded to destination. Classification data backup ID={}'.format(import_request['id']))
+    else:
+        raise click.ClickException("Failed to upload classification file. Destination status: {}".format(
+            response.status_code)
+        )
+
+
+@api.command()
+@click.option('-r', '--retailer')
+@click.option('-m', '--model', )
+@click.option('-i', '--images', type=click.Path(exists=True, file_okay=False, dir_okay=True))
+@click.option('-l', '--labels', type=click.Path(exists=True, file_okay=True, dir_okay=False))
+@click.option('-f', '--features', type=click.Path(exists=True, file_okay=True, dir_okay=False))
+@pass_rebotics_context
+def pack_and_import_classification_db(ctx, retailer, model, images, labels, features):
+    packer = ClassificationDatabasePacker(destination=None)
+    if ctx.verbose:
+        click.echo("Packing from provided values: \n"
+                   "labels: {labels} \n"
+                   "features: {features} \n"
+                   "images: {images}".format(labels=labels, features=features, images=images))
+    try:
+        packed = packer.pack(labels, features, images)
+    except ClassificationDatabaseException as exc:
+        raise click.ClickException(str(exc))
+    import_request = ctx.provider.create_classification_database_import(
+        retailer=retailer, model=model, extension=packer.extension
+    )
+    if ctx.verbose:
+        click.echo("Import request: {}".format(import_request))
+    upload_packed(ctx, import_request, packed)
+
+
+@api.command()
+@click.option('-r', '--retailer')
+@click.option('-m', '--model', )
+@click.argument('filepath', type=click.Path(exists=True, file_okay=True, dir_okay=False))
+@pass_rebotics_context
+def import_classification_db(ctx, retailer, model, filepath):
+    extension = os.path.split(filepath)[-1].split('.')[-1]
+    import_request = ctx.provider.create_classification_database_import(
+        retailer=retailer, model=model, extension=extension
+    )
+    with open(filepath, 'rb') as packed:
+        upload_packed(ctx, import_request, packed)
+
+
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
